# pip install rich

import random
import sys
import time
from pathlib import Path

if not (Path.cwd() / 'dictionaries').exists():
    print("""
        Директории со словарями не существует.
        Создайте директорию dictionaries и поместите в нее следующие словари:
            adjectives.txt - словарь прилагательных;
            nouns.txt - словарь существительных;
            verb.txt - словарь глаголов.
    """)
    sys.exit(0)
path = Path.cwd() / 'dictionaries'
adjectives = [x.strip() for x in open(path / 'adjectives.txt', 'r', encoding='utf-8')]
nouns = [x.strip() for x in open(path / 'nouns.txt', 'r', encoding='utf-8')]
verb = [x.strip() for x in open(path / 'verb.txt', 'r', encoding='utf-8')]
merge_vokab = [*adjectives, *nouns, *verb]
s_vokab = list(set(merge_vokab))


def translit(text: str):
    """
    Замена русских букв на соответствующие им английские.
    """
    symbols = ("абвгдеёжзийклмнопрстуфхцчшщъыьэюяАБВГДЕЁЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЭЮЯ",
               "fpdultygpbqrkvyjghcneahwxioismeuzFPDULTYGPBQRKVYJGHCNEAHWXIOISMEUZ")
    tr = {ord(a): ord(b) for a, b in zip(*symbols)}
    return text.translate(tr)


def choice(wok: str, count_symbol: int, title: str) -> tuple:
    """
    Выполняется транслитерация символов.
    Возвращает количество символов слова определенных в переменной.
    Возвращает оригинальное слово.
    """
    if title == "1":
        return translit(wok.strip().title()[:count_symbol]), wok.strip().title()[:count_symbol], wok.strip().title()
    elif title == "2":
        return translit(wok.strip().upper()[:count_symbol]), wok.strip().upper()[:count_symbol], wok.strip().upper()
    elif title == "3":
        return translit(wok.strip()[:count_symbol]), wok.strip()[:count_symbol], wok.strip()
    else:
        return translit(wok.strip()[:count_symbol]), wok.strip()[:count_symbol], wok.strip()


def generate(count: int, count_words: int, count_symbol: int, title: str) -> list:
    """
    Выбираем случайное слово из словарей. Если слов не больше 4,
    выполняется выборка по трем спискам с глаголами, существительными и прилагательными.
    Если слов больше 4, производится случайная выборка по объединенному списку.
    Полученные слова добавляются в списки, которые объединяются и возвращаются из функции.
    """
    global adjectives, nouns, verb, s_vokab
    words_list = []
    vokab = {0: adjectives, 1: nouns, 2: verb, 3: nouns}
    for k in range(int(count)):
        m = []
        r = []
        o = []
        if 0 < count_words <= 4:
            for i in range(count_words):
                wok = random.choice(tuple(vokab[i]))
                if title == "4":
                    mix, rus, orig = choice(wok, count_symbol, "1") if i == 0 else choice(wok, count_symbol, "3")
                else:
                    mix, rus, orig = choice(wok, count_symbol, title)
                m.append(mix)
                r.append(rus)
                o.append(orig)
        elif count_words > 4:
            for i in range(count_words):
                wok = random.choice(tuple(s_vokab))
                if title == "4":
                    mix, rus, orig = choice(wok, count_symbol, "1") if i == 0 else choice(wok, count_symbol, "3")
                else:
                    mix, rus, orig = choice(wok, count_symbol, title)
                m.append(mix)
                r.append(rus)
                o.append(orig)
        words_list.append(("".join(m), " ".join(r), " ".join(o)))
    return words_list


def print_word(mix: list):
    """
    Печать паролей в терминал.
    """
    print(f"\nСгенерированные парольные фразы\n{'-' * 31}")
    for num, i in enumerate(mix):
        print(f'{num + 1}. {i[0]} | {i[1]} | {i[2]}')


def save_word(mix: list):
    """
    Сохранение паролей в текстовый документ.
    """
    with open('password_list.txt', 'w', encoding='utf-8') as file:
        file.write("Сгенерированные парольные фразы\n\n")
        for num, i in enumerate(mix):
            file.write(f'{num + 1}. {i[0]} | {i[1]} | {i[2]}\n')


def main():
    count_words = input("Введите количество слов в пароле: ")
    count_symbol = input("Введите количество символов каждого слова: ")
    title = input("Каждое слово фразы:\n   "
                  "[1] Начинать с заглавной буквы;\n   "
                  "[2] Все буквы заглавные;\n   "
                  "[3] Все буквы прописные;\n   "
                  "[4] Первая буква заглавная, остальные прописные.\n   "
                  ">>> ")
    count = input("Введите количество паролей для генерации: ")

    t = time.monotonic()
    try:
        mix = generate(int(count), int(count_words), int(count_symbol), title)
        print_word(mix)
        save_word(mix)
        print(f'\nВремя генерации паролей: {time.monotonic() - t:.2f}')
    except ValueError:
        print("Вы ввели неверное значение!")
        sys.exit(0)


if __name__ == "__main__":
    main()
